/*
 * dwepcrack v0.5 [misc.c]
 * by h1kari - (c) Dachb0den Labs 2003
 */

/*
 * Copyright (c) 2003 Dachb0den Labs.
 *      David Hulton <h1kari@dachb0den.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by David Hulton.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY David Hulton AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL David Hulton OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <errno.h>

#include <pcap.h>

#include "dwepcrack.h"

/*
 * get the size of the screen so we know how big to make the progress bar
 */
void
getscrmaxyx(void)
{
  struct winsize ws;

  if(ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) < 0)
  {
    fprintf(stderr, "error: unable to determine screen size: %s",
     strerror(errno));
    exit(2);
  }

  max_lines = ws.ws_row;
  max_cols  = ws.ws_col;
}

/*
 * open up the pcap output file and grab the smallest packet in the file
 */
int
getsmallpkt(char *filename)
{
  int count = 0;
  char errbuf[PCAP_ERRBUF_SIZE];
  u_char *pkt;
  struct pcap_pkthdr h;
  struct wi_rx_frame *wi_h;

  small_len = -1;
  verify_len = -1;

  if((p = pcap_open_offline(filename, errbuf)) == NULL)
  {
    fprintf(stderr, "error: unable to open pcap output file: %s\n", errbuf);
    exit(2);
  }

  while(1)
  {
    if((pkt = (u_char *)pcap_next(p, &h)) == NULL)
      break;

    count++;
    wi_h = (struct wi_rx_frame *)pkt;

    if(!(le16toh(wi_h->wi_status) & WI_STAT_BADCRC) &&
     (le16toh(wi_h->wi_frame_ctl) & WI_FCTL_FTYPE) == WI_FTYPE_DATA)
    {
      if(le16toh(wi_h->wi_dat_len) > MAX_PKT_SIZE ||
       le16toh(wi_h->wi_dat_len) < MIN_PKT_SIZE)
        continue;

      /*
       * if we've already got one small_pkt, grab the next and make sure it
       * isn't the same as the small_pkt.
       */
      if(small_len > 0 && verify_len < 0 && memcmp(small_pkt, pkt + IV_OFF,
       MIN(small_len, le16toh(wi_h->wi_dat_len))) != 0)
      {
        verify_len = le16toh(wi_h->wi_dat_len);
        memcpy(verify_pkt, pkt + IV_OFF, verify_len);
      }
      /* packet should always be < MAX_PKT_SIZE, but just in case */
      else if((le16toh(wi_h->wi_dat_len) < small_len || small_len < 0))
      {
        small_len = le16toh(wi_h->wi_dat_len);
        memcpy(small_pkt, pkt + IV_OFF, small_len);
      }
    }
  }

  if(small_len == verify_len && memcmp(small_pkt, verify_pkt, small_len) == 0)
  {
    fprintf(stderr, "error: not enough packets to verify authenticity of "
     "crack. must have at least 2 packets\n");
    exit(2);
  }

  if(small_len < 0 || verify_pkt < 0)
  {
    fprintf(stderr, "error: unable to find a valid data packet in logfile\n");
    exit(2);
  }

  pcap_close(p);

  return count;
} 

/*
 * print out cracked wep keys
 */
void
report_crack(u_char *keys, int idx, int len)
{
  int i;
  u_char *ptr;

  printf("\n\r     \n"
   "\rwep keys successfully cracked!\n");
  for(i = 0; i < len; i++)
  {
    ptr = keys + (i * WEP_KEY_SIZE);
    printf("\r %d: %02x:%02x:%02x:%02x:%02x %c\n", (len == 1 ? idx : i),
     ptr[0], ptr[1], ptr[2], ptr[3], ptr[4], (i == idx ? '*' : ' '));
  }
  printf("\rdone.");
}
