/*
 * dinject v0.1 [dinject-beacon.c]
 * by h1kari - (c) Dachb0den Labs 2002
 */                                                                             

/*                                                                               * Copyright (c) 2002 Dachb0den Labs.
 *      David Hulton <h1kari@dachb0den.com>.  All rights reserved.               *
 * Redistribution and use in source and binary forms, with or without            * modification, are permitted provided that the following conditions
 * are met:                                                                      * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.              * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by David Hulton.                * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission. 
 *                                                                               * THIS SOFTWARE IS PROVIDED BY David Hulton AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE         * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL David Hulton OR THE VOICES IN HIS HEAD     * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF          * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <libwnet.h>

#include "dinject.h"

#define BANNER \
 "* dinject-beacon v0.1 by h1kari <h1kari@dachb0den.com> *\n" \
 "* Copyright (c) Dachb0den Labs 2002 [http://dachb0den.com] *\n"

extern int flags;

void
usage(char *progname)
{
  printf("\n%s\n", BANNER);
  printf("usage: %s <device> [-v] [options]\n\n", progname);
  _80211_usage();
  beacon_usage();
  ssid_usage();
  srates_usage();
  fhparm_usage();
  dsparm_usage();
  cfparm_usage();
  ibssparm_usage();
  tim_usage();
  printf("\n");

  exit(2);
}

#define PKT_SIZE 4096
int
main(int argc, char *argv[])
{
  int c, off = 0;
  char *dev;
  u_char *pkt;
  struct wi_frame _80211_hdr;
  struct wi_mgmt_beacon_hdr beacon_hdr;
  struct wi_mgmt_var_hdr ssid_hdr, srates_hdr, fhparm_hdr, dsparm_hdr;
  struct wi_mgmt_var_hdr cfparm_hdr, ibssparm_hdr, tim_hdr;
  flags = USE_SSID | USE_SRATES;

  if (argc < 2)
    usage(argv[0]);

  dev = argv[1];

  _80211_hdr = _80211_defaults();
  beacon_hdr = beacon_defaults();
  ssid_hdr = ssid_defaults();
  srates_hdr = srates_defaults();
  fhparm_hdr = fhparm_defaults();
  dsparm_hdr = dsparm_defaults();
  cfparm_hdr = cfparm_defaults();
  ibssparm_hdr = ibssparm_defaults();
  tim_hdr = tim_defaults();

  _80211_hdr.wi_frame_ctl |= IEEE80211_FC0_TYPE_MGT |
   IEEE80211_FC0_SUBTYPE_BEACON;
  verbose = 0;

  while ((c = getopt(argc - 1, argv + 1, BASE_OPT _80211_OPT BEACON_OPT SSID_OPT
   SRATES_OPT FHPARM_OPT DSPARM_OPT CFPARM_OPT TIM_OPT IBSSPARM_OPT)) != EOF)
  {
    switch(c)
    {
      BASE_SETOPT()
      _80211_SETOPT(&_80211_hdr)
      BEACON_SETOPT(&beacon_hdr)
      SSID_SETOPT(&ssid_hdr)
      SRATES_SETOPT(&srates_hdr)
      FHPARM_SETOPT(&fhparm_hdr)
      DSPARM_SETOPT(&dsparm_hdr)
      CFPARM_SETOPT(&cfparm_hdr)
      TIM_SETOPT(&tim_hdr)
      IBSSPARM_SETOPT(&ibssparm_hdr)
      default:
        break;
    }
  }

  pkt = (u_char *)malloc(PKT_SIZE);

  off += _80211_WRITE(_80211_hdr, pkt + off, PKT_SIZE - off);
  off += BEACON_WRITE(beacon_hdr, pkt + off, PKT_SIZE - off);

  if(flags & USE_SSID)
  {
    off += SSID_WRITE(ssid_hdr, pkt + off, PKT_SIZE - off);
  }

  if(flags & USE_SRATES)
  {
    off += SRATES_WRITE(srates_hdr, pkt + off, PKT_SIZE - off);
  }

  if(flags & USE_FHPARM)
  {
    off += FHPARM_WRITE(fhparm_hdr, pkt + off, PKT_SIZE - off);
  }

  if(flags & USE_DSPARM)
  {
    off += DSPARM_WRITE(dsparm_hdr, pkt + off, PKT_SIZE - off);
  }

  if(flags & USE_CFPARM)
  {
    off += CFPARM_WRITE(cfparm_hdr, pkt + off, PKT_SIZE - off);
  }

  if(flags & USE_TIM)
  {
    off += TIM_WRITE(tim_hdr, pkt + off, PKT_SIZE - off);
  }

  if(flags & USE_IBSSPARM)
  {
    off += IBSSPARM_WRITE(ibssparm_hdr, pkt + off, PKT_SIZE - off);
  }

  _80211_INJECT(dev, pkt, off);

  exit(0);
}
